<?php
/**
 * Class Medias ToolBox Admin
 * The admin-specific functionality of the plugin
 *
 * @author NeosLab <contact@neoslab.com>
 * @link https://neoslab.com
 * @version 1.5.0
 * @package Medias_ToolBox
*/
class Medias_ToolBox_Admin
{
	/**
	 * The ID of this plugin
	*/
	private $plugin_name;

	/**
	 * The version of this plugin
	*/
	private $version;

	/**
	 * Initialize the class and set its properties
	*/
	public function __construct($plugin_name, $version)
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the dashboard area
	*/
	public function enqueue_backend_standalone()
	{
		wp_register_style($this->plugin_name.'-standalone', plugin_dir_url(__FILE__).'assets/styles/standalone.css', array(), $this->version, 'all');
		wp_enqueue_style($this->plugin_name.'-standalone');
	}

	/**
	 * Register the stylesheets for the admin area
	*/
	public function enqueue_backend_styles()
	{
		wp_register_style($this->plugin_name.'-css-fontawesome', plugin_dir_url(__FILE__).'assets/styles/fontawesome.min.css', array(), $this->version, 'all');
		wp_register_style($this->plugin_name.'-css-dashboard', plugin_dir_url(__FILE__).'assets/styles/dashboard.css', array(), $this->version, 'all');
		wp_enqueue_style($this->plugin_name.'-css-fontawesome');
		wp_enqueue_style($this->plugin_name.'-css-dashboard');
	}

	/**
	 * Register the JavaScript for the admin area
	*/
	public function enqueue_backend_scripts()
	{
		wp_register_script($this->plugin_name.'-js-script', plugin_dir_url(__FILE__).'assets/javascripts/script.js', array('jquery'), $this->version, false);
		wp_enqueue_script($this->plugin_name.'-js-script');
	}

	/**
	 * Return the plugin header
	*/
	public function return_plugin_header()
	{
		$html = '<div class="header-plugin"><span class="header-icon"><i class="fas fa-sliders-h"></i></span> <span class="header-text">'.__('Medias Toolbox', 'medias-toolbox').'</span></div>';
		return $html;
	}

	/**
	 * Return the tabs menu
	*/
	public function return_tabs_menu($tab)
	{
		$link = admin_url('admin.php');
		$list = array
		(
			array('tab1', 'medias-toolbox-compress', 'fa-compress', __('Compress', 'medias-toolbox')),
			array('tab2', 'medias-toolbox-renamer', 'fa-edit', __('Renamer', 'medias-toolbox')),
			array('tab3', 'medias-toolbox-about', 'fa-info-circle', __('About', 'medias-toolbox'))
		);

		foreach($list as $item => $value)
		{
			$menu ='<div class="tab-label '.$value[0].' '.(($tab == $value[0]) ? 'active' : '').'"><a href="'.$link.'?page='.$value[1].'"><i class="fas '.$value[2].'"></i> <span>'.$value[3].'</span></a></div>';
			echo $menu;
		}
	}

	/**
	 * Check file extension
	*/
	private function return_file_type($string)
	{
		$i = strrpos($string,".");

		if(!$i)
		{
			return "";
		}

		$case = strlen($string) - $i;
		$exts = substr($string, $i+1, $case);
		return strtolower($exts);
	}

	/**
	 * Return String Format Date + Name
	*/
	private function return_string_date_name($string)
	{
		return date('YmdHi').'-'.sanitize_file_name($string);
	}

	/**
	 * Return String Format Time + Name
	*/
	private function return_string_time_name($string)
	{
		return time().'-'.sanitize_file_name($string);
	}

	/**
	 * Return String Format Date + Rand ID
	*/
	private function return_string_date_randid()
	{
		$l = 50;
		$c = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

		$s = "";
		for($p = 0; $p < $l; $p++)
		{
		    $s.= $c[mt_rand(0, strlen($c)-1)];
		}

		return date('YmdHi').'-'.($s);
	}

	/**
	 * Return String Format Time + Rand ID
	*/
	private function return_string_time_randid()
	{
		$l = 50;
		$c = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';

		$s = "";
		for($p = 0; $p < $l; $p++)
		{
		    $s.= $c[mt_rand(0, strlen($c)-1)];
		}

		return time().'-'.($s);
	}

	/**
	 * Return String Format Date + Md5(Rand)
	*/
	private function return_string_date_md5rand()
	{
		return date('YmdHi').'-'.md5(uniqid(rand(), true));
	}

	/**
	 * Return String Format Time + Md5(Rand)
	*/
	private function return_string_time_md5rand()
	{
		return time().'-'.md5(uniqid(rand(), true));
	}

	/**
	 * Execute the rename function during upload
	*/
	public function return_rename_upload($fileName)
	{
		$opts = get_option('_medias_toolbox_renamer');

		if((isset($opts['renamer'])) && ($opts['renamer'] == 'on'))
		{
			$exts = $this->return_file_type($fileName);
			$name = basename($fileName, $exts);
			$name = rtrim($name, '.');

			if(((isset($exts)) && (in_array($exts, MTB_PLUGIN_FILETYPE))) && ((isset($opts[$exts])) && ($opts[$exts] == 'on')))
			{
				if(isset($opts['pattern']))
				{
					if($opts['pattern'] == '1')
					{
						$fileName = $this->return_string_date_name($name).'.'.$exts;
					}
					elseif($opts['pattern'] == '2')
					{
						$fileName = $this->return_string_time_name($name).'.'.$exts;
					}
					elseif($opts['pattern'] == '3')
					{
						$fileName = $this->return_string_date_randid().'.'.$exts;
					}
					elseif($opts['pattern'] == '4')
					{
						$fileName = $this->return_string_time_randid().'.'.$exts;
					}
					elseif($opts['pattern'] == '5')
					{
						$fileName = $this->return_string_date_md5rand().'.'.$exts;
					}
					elseif($opts['pattern'] == '6')
					{
						$fileName = $this->return_string_time_md5rand().'.'.$exts;
					}
					else
					{
						$fileName = $this->return_string_date_name($name).'.'.$exts;
					}
				}
				else
				{
					$fileName = $this->return_string_date_name($name).'.'.$exts;
				}
			}
		}

		return $fileName;
	}

	/**
	 * Create Record Database
	*/
	public function create_record_database()
	{
		global $wpdb;

		$table = $wpdb->prefix.'medias';
		if($wpdb->get_var("SHOW TABLES LIKE '$table'") !== $table)
		{
			$charset = $wpdb->get_charset_collate();
			$query = "CREATE TABLE $table(
			id bigint(20) NOT NULL AUTO_INCREMENT,
			file varchar(255) DEFAULT '' NOT NULL,
			size varchar(10) DEFAULT '' NOT NULL,
			stat int(1) DEFAULT '0' NOT NULL,
			PRIMARY KEY (id)
			) $charset;";
			$fetch = $wpdb->query($query);
		}
	}

	/**
	 * Compress Image
	*/
	public function compress_image($source_url, $destination_url, $quality)
	{
		$info = getimagesize($source_url);
		if($info['mime'] == 'image/jpeg')
		{
			$image = imagecreatefromjpeg($source_url);
		}
		elseif($info['mime'] == 'image/gif')
		{
			$image = imagecreatefromgif($source_url);
		}
		elseif($info['mime'] == 'image/png')
		{
			$image = imagecreatefrompng($source_url);
		}

		imagejpeg($image, $destination_url, $quality);
		return $destination_url;
	}

	/**
	 * Compress Media
	*/
	public function return_proceed_medias()
	{
		global $wpdb;

		if((isset($_GET['page'])) && ($_GET['page'] == 'medias-toolbox-compress'))
		{
			if((isset($_GET['paged'])) && (is_numeric($_GET['paged'])))
			{
				$paged = $_GET['paged'];
			}
			else
			{
				$paged = '1';
			}

			if((isset($_GET['file'])) && ((isset($_GET['action'])) && ($_GET['action'] == 'compress')))
			{
				$file = urldecode($_GET['file']);
				if(file_exists($file))
				{
					$path = substr($file, 0, strrpos($file, "/"));
					$this->compress_image($file, $path.'/'.basename($file), 70);

					$table = $wpdb->prefix.'medias';
					$query = "UPDATE `".$table."` SET `stat` = '1' WHERE `file` = '".$file."'";
					$fetch = $wpdb->query($query);

					header('location:'.admin_url('admin.php?page=medias-toolbox-compress').'&status=compressed&paged='.$paged);
					die();
				}
			}

            if(isset($_GET['thumb']))
            {
                foreach($_GET['thumb'] as $file)
                {
                    $file = urldecode($file);
                    if(file_exists($file))
                    {
                        $path = substr($file, 0, strrpos($file, "/"));
                        $this->compress_image($file, $path.'/'.basename($file), 70);

                        $table = $wpdb->prefix.'medias';
						$query = "UPDATE `".$table."` SET `stat` = '1' WHERE `file` = '".$file."'";
						$fetch = $wpdb->query($query);
                    }
                }


                header('location:'.admin_url('admin.php?page=medias-toolbox-compress').'&status=compressed&paged='.$paged);
                die();
            }
		}
	}

	/**
	 * Update `Options` on form submit
	*/
	public function return_update_options()
	{
		if((isset($_POST['mtb-update-renamer'])) && ($_POST['mtb-update-renamer'] == 'true')
		&& check_admin_referer('mtb-referer-form', 'mtb-referer-renamer'))
		{
			$opts = array('renamer' => 'off', 'pattern' => '1');

			if(isset($_POST['_medias_toolbox_renamer']['renamer']))
			{
				$opts['renamer'] = 'on';
			}

			foreach(MTB_PLUGIN_FILETYPE as $fileExt)
			{
				if(isset($_POST['_medias_toolbox_renamer'][$fileExt]))
				{
					$opts[$fileExt] = 'on';
				}
			}

			$allowed = array('1', '2', '3', '4', '5', '6');
			if((isset($_POST['_medias_toolbox_renamer']['pattern']))
			&& (in_array($_POST['_medias_toolbox_renamer']['pattern'], $allowed)))
			{
				$opts['pattern'] = sanitize_text_field($_POST['_medias_toolbox_renamer']['pattern']);
			}

			$data = update_option('_medias_toolbox_renamer', $opts);
			header('location:'.admin_url('admin.php?page=medias-toolbox-renamer').'&status=updated');
			die();
		}
	}

	/**
	 * Return the `Compress` page
	*/
	public function return_compress_page()
	{
		if(((isset($_GET['page'])) && ($_GET['page'] == 'medias-toolbox-compress')) && ((isset($_GET['status'])) && ($_GET['status'] == 'compressed')))
		{
			$notice = array('success', __('The selected media have been successfully compressed !!', 'medias-toolbox'));
		}

		?>
		<div class="wrap">
			<section class="wpbnd-wrapper">
				<div class="wpbnd-container">
					<div class="wpbnd-tabs">
						<?php echo $this->return_plugin_header(); ?>
						<main class="tabs-main">
							<?php echo $this->return_tabs_menu('tab1'); ?>
							<section class="tab-section">
								<?php if(isset($notice)) { ?>
								<div class="wpbnd-notice <?php echo $notice[0]; ?>">
									<span class="close-btn" onclick="this.parentElement.style.display='none';">&times;</span>
									<span><?php echo _e($notice[1], 'medias-toolbox'); ?></span>
								</div>
								<?php } else { ?>
								<div class="wpbnd-notice info">
									<span class="close-btn" onclick="this.parentElement.style.display='none';">&times;</span>
									<span><?php echo _e('Below the list of all the images that currently present in your database.', 'medias-toolbox'); ?></span>
								</div>
								<?php } ?>
								<div class="wpbnd-datatables">
									<?php
									$records = $this->create_record_database();
								    if(!class_exists('WP_List_Table'))
									{
									    require_once(ABSPATH.'wp-admin/includes/class-wp-list-table.php');
									}

									require_once plugin_dir_path(dirname(__FILE__)).'admin/class-medias-toolbox-table.php';
									$listTable = new Medias_ToolBox_Table();
								    $listTable->prepare_items();
								    ?>
								    <form id="images-filter" method="GET">
							            <input type="hidden" name="page" value="<?php echo $_REQUEST['page'] ?>" />
							            <?php $listTable->display() ?>
							        </form>
							    </div>
							</section>
						</main>
					</div>
				</div>
			</section>
		</div>
		<?php
	}

	/**
	 * Return the `Renamer` page
	*/
	public function return_renamer_page()
	{
		$opts = get_option('_medias_toolbox_renamer');

		if((isset($_GET['status'])) && ($_GET['status'] == 'updated'))
		{
			$notice = array('success', __('Your settings have been successfully updated.', 'medias-toolbox'));
		}

		?>
		<div class="wrap">
			<section class="wpbnd-wrapper">
				<div class="wpbnd-container">
					<div class="wpbnd-tabs">
						<?php echo $this->return_plugin_header(); ?>
						<main class="tabs-main">
							<?php echo $this->return_tabs_menu('tab2'); ?>
							<section class="tab-section">
								<?php if(isset($notice)) { ?>
								<div class="wpbnd-notice <?php echo $notice[0]; ?>">
									<span class="close-btn" onclick="this.parentElement.style.display='none';">&times;</span>
									<span><?php echo _e($notice[1], 'medias-toolbox'); ?></span>
								</div>
								<?php } elseif((isset($opts['renamer']) && ($opts['renamer']) == 'off')) { ?>
								<div class="wpbnd-notice warning">
									<span class="close-btn" onclick="this.parentElement.style.display='none';">&times;</span>
									<span><?php echo _e('You have not set up your automatic rename options ! In order to do so, please use the below form.', 'medias-toolbox'); ?></span>
								</div>
								<?php } else { ?>
								<div class="wpbnd-notice info">
									<span class="close-btn" onclick="this.parentElement.style.display='none';">&times;</span>
									<span><?php echo _e('Your plugin is properly configured ! You can change at anytime your automatic rename options using the below form.', 'medias-toolbox'); ?></span>
								</div>
								<?php } ?>
								<form method="POST">
									<input type="hidden" name="mtb-update-renamer" value="true" />
									<?php wp_nonce_field('mtb-referer-form', 'mtb-referer-renamer'); ?>
									<div class="wpbnd-form">
										<div class="field">
											<?php $fieldID = uniqid(); ?>
											<span class="label"><?php echo _e('Automatic Files Renamer', 'medias-toolbox'); ?></span>
										    <div class="onoffswitch">
										        <input id="<?php echo $fieldID; ?>" type="checkbox" name="_medias_toolbox_renamer[renamer]" class="onoffswitch-checkbox input-renamer" <?php if((isset($opts['renamer'])) && ($opts['renamer'] == 'on')) { echo 'checked="checked"';} ?>/>
										        <label class="onoffswitch-label" for="<?php echo $fieldID; ?>">
										            <span class="onoffswitch-inner"></span>
										            <span class="onoffswitch-switch"></span>
										        </label>
										    </div>
										    <small><?php echo _e('Do you want to automatically rename your files on upload ?', 'medias-toolbox'); ?></small>
										</div>
										<div id="handler-renamer" class="subfield <?php if((isset($opts['renamer'])) && ($opts['renamer'] == 'on')) { echo 'show'; } ?>">
											<?php
											foreach(MTB_PLUGIN_FILETYPE as $fileExt)
											{
												$fieldID = uniqid();
												$output = '<div class="field">';
												$output.= '<span class="label">'.sprintf(__('Rename %s File', 'medias-toolbox'), strtoupper($fileExt)).'</span>';
												$output.= '<div class="onoffswitch">';
												$output.= '<input id="'.$fieldID.'" type="checkbox" name="_medias_toolbox_renamer['.$fileExt.']" class="onoffswitch-checkbox" '.(((isset($opts[$fileExt])) && ($opts[$fileExt] == 'on')) ? 'checked="checked"' : '').'/>';
											    $output.= '<label class="onoffswitch-label" for="'.$fieldID.'">';
												$output.= '<span class="onoffswitch-inner"></span>';
												$output.= '<span class="onoffswitch-switch"></span>';
												$output.= '</label>';
												$output.= '</div>';
											    $output.= '<small>'.sprintf(__('Do you want to automatically rename your %s files on upload ?', 'medias-toolbox'), strtoupper($fileExt)).'</small>';
												$output.= '</div>';

												echo $output;
											}

											?>
											<div class="field">
												<span class="label"><?php echo _e('Rename Patterns', 'medias-toolbox'); ?></span>
												<?php
												$level = array
												(
													'1' => array
													(
														'title' => __('Current Date/Time + Sanitized filename', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-original-filename-sanitized.jpg'), date('YmdHi'))
													),
													'2' => array
													(
														'title' => __('Current Timestamp + Sanitized filename', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-original-filename-sanitized.jpg'), time())
													),
													'3' => array
													(
														'title' => __('Current Date/Time + Unique Random String', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-%s.jpg'), date('YmdHi'), md5('original-filename-sanitized'))
													),
													'4' => array
													(
														'title' => __('Current Timestamp + Unique Random String', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-%s.jpg'), time(), md5('original-filename-sanitized'))
													),
													'5' => array
													(
														'title' => __('Current Date/Time + Encrypted filename', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-%s.jpg'), date('YmdHi'), md5('original-filename-sanitized'))
													),
													'6' => array
													(
														'title' => __('Current Timestamp + Encrypted filename', 'medias-toolbox'),
														'pattern' => sprintf(__('%s-%s.jpg'), time(), md5('original-filename-sanitized'))
													)
												);

												if(!isset($opts['pattern']))
												{
													$opts = array('pattern' => '1');
												}

												foreach($level as $minkey => $minval)
												{
													$fieldID = uniqid();
													$output = '<div class="break-05"><!-- Line Break 05px --></div>';
													$output.= '<label class="radiobox">';
													$output.= '<span><b>'.$minval["title"].'</b><br>'.$minval["pattern"].'</span>';
													$output.= '<input type="radio" id="'.$fieldID.'" name="_medias_toolbox_renamer[pattern]" value="'.$minkey.'" class="common" '.(((isset($opts['pattern'])) && ($opts['pattern'] == $minkey)) ? 'checked="checked"' : '').' />';
													$output.= '<span style="top:10px;" class="checkmark"></span>';
													$output.= '</label>';
													$output.= '<hr/>';
													echo $output;
												}

												?>
											</div>
										</div>
										<div class="form-footer">
											<input type="submit" class="button button-primary button-theme" style="height:45px;" value="<?php _e('Update Settings', 'medias-toolbox'); ?>">
										</div>
									</div>
								</form>
							</section>
						</main>
					</div>
				</div>
			</section>
		</div>
		<?php
	}

	/**
	 * Return the `About` page
	*/
	public function return_about_page()
	{
		?>
		<div class="wrap">
			<section class="wpbnd-wrapper">
				<div class="wpbnd-container">
					<div class="wpbnd-tabs">
						<?php echo $this->return_plugin_header(); ?>
						<main class="tabs-main">
							<?php echo $this->return_tabs_menu('tab3'); ?>
							<section class="tab-section">
								<div class="about-wrapper">
									<div class="about-plugins"><h4><i class="fas fa-laptop-code"></i> <?php echo _e('Discover all our available plugins', 'medias-toolbox'); ?></h4></div>
									<table class="plugins-table">
										<tbody>
											<?php
											$items = array
											(
												array
												(
													'name' => 'Custom Email Sender',
													'base' => 'custom-email-sender',
													'vers' => '2.2.6',
													'date' => '2021-08-25',
													'test' => '5.8',
													'lang' => 'English, French, Spanish'
												),
												array
												(
													'name' => 'Database Toolset',
													'base' => 'database-toolset',
													'vers' => '1.5.6',
													'date' => '2021-08-25',
													'test' => '5.8',
													'lang' => 'English, French, Spanish'
												),
												array
												(
													'name' => 'Medias Toolbox',
													'base' => 'medias-toolbox',
													'vers' => '1.5.0',
													'date' => '2021-08-25',
													'test' => '5.8',
													'lang' => 'English, French, Spanish'
												),
												array
												(
													'name' => 'Powered Minifier',
													'base' => 'powered-minifier',
													'vers' => '1.5.0',
													'date' => '2021-08-25',
													'test' => '5.8',
													'lang' => 'English, French, Spanish'
												)
											);

											foreach($items as $item)
											{
												$table = '<tr>';
												$table.= '<td width="160px" scope="col"><a class="thumb" ref="nofollow noreferrer noopener" target="_blank" href="https://wordpress.org/plugins/'.$item['base'].'/"><img src="https://ps.w.org/'.$item['base'].'/assets/icon-256x256.png" alt="logo"/></a></td>';
												$table.= '<td scope="col">';
												$table.= '<ul class="plugin-details">';
												$table.= '<li>'.__('Plugin', 'polygon').' : <strong>'.$item['name'].'</strong></li>';
												$table.= '<li>'.__('Version', 'polygon').' : <strong>'.$item['vers'].'</strong></li>';
												$table.= '<li>'.__('Last release', 'polygon').' : <strong>'.$item['date'].'</strong></li>';
												$table.= '<li>'.__('Wordpress Version', 'polygon').' : <strong>'.$item['test'].'</strong></li>';
												$table.= '<li>'.__('Languages', 'polygon').' : <strong>'.$item['lang'].'</strong></li>';
												$table.= '<li><a ref="nofollow noreferrer noopener" target="_blank" href="https://wordpress.org/plugins/'.$item['base'].'/">'.__('Visit on Wordpress.org', 'polygon').'</a></li>';
												$table.= '</ul>';
												$table.= '</td>';
												$table.= '</tr>';
												echo $table;
											}
											?>
										</tbody>
									</table>
								</div>
							</section>
						</main>
					</div>
				</div>
			</section>
		</div>
		<?php
	}

	/**
	 * Return Backend Menu
	*/
	public function return_admin_menu()
	{
		add_menu_page('Medias ToolBox', 'Medias ToolBox', 'administrator', 'medias-toolbox-admin', array($this, 'return_compress_page'), 'dashicons-format-image');
		add_submenu_page('medias-toolbox-admin', 'Clean-Up', 'Medias Compress', 'administrator', 'medias-toolbox-compress', array($this, 'return_compress_page'));
		add_submenu_page('medias-toolbox-admin', 'Renamer', 'Medias Renamer', 'administrator', 'medias-toolbox-renamer', array($this, 'return_renamer_page'));
		add_submenu_page('medias-toolbox-admin', 'About', 'About', 'administrator', 'medias-toolbox-about', array($this, 'return_about_page'));
		remove_submenu_page('medias-toolbox-admin', 'medias-toolbox-admin');
	}
}

?>
