<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://neoslab.com
 * @since             1.5.0
 * @package           Medias_ToolBox
 *
 * @wordpress-plugin
 * Plugin Name: Medias ToolBox
 * Plugin URI: https://neoslab.com/plugins/medias-toolbox/
 * Description: Medias Toolbox allow you to sanitize and rename automatically media files during upload. Furthermore it also offer you the option to compress your JPG and PNG image file to optimize your website performance.
 * Version: 1.5.0
 * Author: NeosLab
 * Author URI: https://neoslab.com
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain: medias-toolbox
 * Domain Path: /languages
*/

/**
 * Prevent file to be called directly
*/
if((!defined('WPINC')) || ('medias-toolbox.php' == basename($_SERVER['SCRIPT_FILENAME'])))
{
	die;
}

/**
 * Define Constants
*/
define('MTB_PLUGIN_FULLNAME', 'Medias ToolBox');
define('MTB_PLUGIN_IDENTIFIER', 'medias-toolbox');
define('MTB_PLUGIN_VERSION', '1.5.0');
define('MTB_PLUGIN_LAST_RELEASE', '2021/06/13');
define('MTB_PLUGIN_LANGUAGES', 'English, French, Spanish');

/**
 * Define Allowed Filetype
*/

$fileType = array
(
	"jpg", "jpeg", "png", "gif",
	"pdf", "doc", "docx", "ppt",
	"pptx", "pps", "ppsx", "odt",
	"xls", "xlsx", "mp3", "m4a",
	"ogg", "wav", "mp4", "m4v",
	"mov", "wmv", "avi", "mpg",
	"ogv", "3gp", "3g2", "zip",
	"tar", "bzip2", "7z", "rar"
);

define('MTB_PLUGIN_FILETYPE', $fileType);

/**
 * The code that runs during plugin activation
 * This action is documented in includes/class-medias-toolbox-activation.php
*/
if(!function_exists('activate_medias_toolbox'))
{
	function activate_medias_toolbox()
	{
		require_once plugin_dir_path(__FILE__).'includes/class-medias-toolbox-activation.php';
		Medias_ToolBox_Activation::activate();
	}

	register_activation_hook(__FILE__, 'activate_medias_toolbox');
}

/**
 * The code that runs during plugin deactivation
 * This action is documented in includes/class-medias-toolbox-deactivation.php
*/
if(!function_exists('deactivate_medias_toolbox'))
{
	function deactivate_medias_toolbox()
	{
		require_once plugin_dir_path(__FILE__).'includes/class-medias-toolbox-deactivation.php';
		Medias_ToolBox_Deactivation::deactivate();
	}

	register_deactivation_hook(__FILE__, 'deactivate_medias_toolbox');
}

/**
 * The core plugin class that is used to define internationalization
 * admin-specific hooks and public-facing site hooks
*/
require plugin_dir_path(__FILE__).'includes/class-medias-toolbox-core.php';

/**
 * Begins execution of the plugin
*/
if(!function_exists('run_medias_toolbox'))
{
	function run_medias_toolbox()
	{
		$plugin = new Medias_ToolBox_Core();
		$plugin->run();
	}

	run_medias_toolbox();
}

?>
